"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vdom_utils_1 = require("vdom-utils");
const utils_1 = require("./utils");
const defaultOpts = {
    sort: false,
    shallow: false,
    depth: 0,
};
exports.createRenderer = (renderer, options = {}) => {
    const opts = Object.assign({}, defaultOpts, options);
    if (opts.depth === undefined) {
        opts.depth = defaultOpts.depth;
    }
    return (vnode) => {
        renderer.reset();
        renderToString(vnode, renderer, opts.depth, opts);
        return renderer.html;
    };
};
function renderToString(vnode, renderer, depth, options) {
    if (vnode === undefined) {
        return;
    }
    // Text node
    if (typeof vnode === "string") {
        renderer.onTextNode(vdom_utils_1.escapeAttr(vnode), depth);
        return;
    }
    const { shallow, sort } = options;
    const { attributes } = vnode;
    let { nodeName, children } = vnode;
    // Component
    if (typeof nodeName === "function") {
        if (shallow) {
            nodeName = utils_1.getComponentName(nodeName);
        }
        else {
            // Stateless
            let rendered;
            const props = utils_1.getNodeProps(vnode);
            if (!nodeName.prototype ||
                typeof nodeName.prototype.render !== "function") {
                rendered = nodeName(props, undefined);
            }
            else {
                // Class components
                const c = new nodeName(props, undefined);
                if (c.componentWillMount !== undefined) {
                    c.componentWillMount();
                }
                rendered = c.render(props, undefined);
            }
            renderToString(rendered, renderer, depth, options);
            return;
        }
    }
    const hasAttributes = attributes !== undefined;
    if (hasAttributes && vnode.attributes.children !== undefined) {
        children = vnode.attributes.children;
    }
    const isVoid = vdom_utils_1.VOID_ELEMENTS.includes(nodeName) ||
        (shallow && children.length === 0);
    renderer.onOpenTag(nodeName, hasAttributes, isVoid, depth);
    let dangerHtml;
    if (hasAttributes) {
        let keys = Object.keys(attributes);
        if (sort) {
            keys = keys.sort();
        }
        const keyLen = keys.length;
        for (var i = 0; i < keyLen; i++) {
            let name = keys[i];
            let value = attributes[name];
            if (value === undefined ||
                value === null ||
                value === false ||
                name === "children" ||
                name === "key" ||
                name === "ref" ||
                typeof value === "function") {
                continue;
            }
            else if (name === "className") {
                if (attributes.class !== undefined) {
                    continue;
                }
                name = "class";
            }
            else if (name === "style") {
                let styles = "";
                const props = Object.keys(value);
                const styleLen = props.length;
                for (var k = 0; k < styleLen; k++) {
                    const prop = props[k];
                    styles +=
                        vdom_utils_1.escapeAttr(vdom_utils_1.jsToCss(prop)) + ": " + vdom_utils_1.escapeAttr("" + value[prop]) + ";";
                    if (k !== styleLen - 1) {
                        styles += " ";
                    }
                }
                value = styles;
            }
            else if (name === "dangerouslySetInnerHTML") {
                dangerHtml = value.__html;
                continue;
            }
            if (name.startsWith("xlink")) {
                name = name.toLowerCase().replace(/^xlink\:?(.+)/, "xlink:$1");
            }
            if (typeof value === "string") {
                value = vdom_utils_1.escapeAttr(value);
            }
            renderer.onProp(name, value, depth);
        }
    }
    renderer.onOpenTagClose(nodeName, hasAttributes, isVoid, children.length > 0, depth);
    if (dangerHtml !== undefined) {
        renderer.onDangerousInnerHTML(dangerHtml);
    }
    else {
        const childrenLen = children.length;
        if (childrenLen > 0) {
            for (var j = 0; j < childrenLen; j++) {
                renderToString(children[j], renderer, depth + 1, options);
            }
        }
    }
    renderer.onCloseTag(nodeName, isVoid, depth);
}
exports.renderToString = renderToString;
//# sourceMappingURL=renderSync.js.map